@@ -23,7 +23,7 @@ auto_exts = c(
 dev2ext = function(x) {
   res = auto_exts[x]
   if (any(idx <- is.na(res))) {
-    for (i in x[idx]) check_dev(i)
+    for (i in x[idx]) dev_get(i)
     stop2(
       'cannot find appropriate filename extensions for device ', x[idx], '; ',
       "please use chunk option 'fig.ext' (https://yihui.org/knitr/options)"
@@ -32,56 +32,24 @@ dev2ext = function(x) {
   unname(res)
 }
 
-check_dev = function(dev) {
-  if (exists(dev, mode = 'function', envir = knit_global()))
-    get(dev, mode = 'function', envir = knit_global()) else
-      stop('the graphical device', sQuote(dev), 'does not exist (as a function)')
-}
-
-# quartiz devices under Mac
-quartz_dev = function(type, dpi) {
-  force(type); force(dpi)
-  function(file, width, height, ...) {
-    grDevices::quartz(file = file, width = width, height = height, type = type, dpi = dpi, ...)
+# test if a device is available (and remember it), e.g., dev_available('png',
+# png); capabilities() gives similar results for some devices but this function
+# is more general
+dev_available = local({
+  res = list()
+  function(name, fun = dev_get(name)) {
+    if (!is.null(res[[name]])) return(res[[name]])
+    res[[name]] <<- tryCatch({
+      f = tempfile(); on.exit(unlink(f))
+      fun(f, width = 5, height = 5)
+      grDevices::dev.off()
+      TRUE
+    }, error = function(e) FALSE)
   }
-}
+})
 
-# a wrapper of the tikzDevice::tikz device
-tikz_dev = function(...) {
-  loadNamespace('tikzDevice')
-  packages = switch(
-    getOption('tikzDefaultEngine'),
-    pdftex = getOption('tikzLatexPackages'),
-    xetex = getOption('tikzXelatexPackages'),
-    luatex = getOption('tikzLualatexPackages')
-  )
-  tikzDevice::tikz(..., packages = c('\n\\nonstopmode\n', packages, .knitEnv$tikzPackages))
-}
-
-# a wrapper of the ragg::agg_png device
-ragg_png_dev = function(...) {
-  loadNamespace('ragg')
-  args = list(...)
-  # handle bg -> background gracefully
-  args$background = args$background %n% args$bg
-  args$bg = NULL
-  do.call(ragg::agg_png, args)
-}
-
-# save a recorded plot
-save_plot = function(plot, name, dev, width, height, ext, dpi, options) {
-
-  path = paste(name, ext, sep = '.')
-  # when cache=2 and plot file exists, just return the filename
-  if (options$cache == 2 && cache$exists(options$hash, options$cache.lazy)) {
-    if (in_base_dir(!file.exists(path))) {
-      purge_cache(options)
-      stop('cannot find ', path, '; the cache has been purged; please re-compile')
-    }
-    return(paste(name, if (dev == 'tikz' && options$external) 'pdf' else ext, sep = '.'))
-  }
-
-  # built-in devices
+dev_get = function(dev, options = opts_current$get(), dpi = options$dpi[1]) {
+  if (is.null(dpi) || is.na(dpi)) dpi = 72
   device = switch(
     dev,
     bmp = function(...) bmp(...,  res = dpi, units = 'in'),
@@ -127,10 +95,58 @@ save_plot = function(plot, name, dev, width, height, ext, dpi, options) {
 
     tikz = function(...) {
       tikz_dev(..., sanitize = options$sanitize, standAlone = options$external)
-    },
-
-    check_dev(dev)
+    }
   )
+  if (!is.null(device)) return(device)
+  # custom device provided by user as a character string
+  if (!exists(dev, mode = 'function', envir = knit_global()))
+    stop('The graphical device ', sQuote(dev), ' was not found (as a function).')
+  get(dev, mode = 'function', envir = knit_global())
+}
+
+# quartz devices under Mac
+quartz_dev = function(type, dpi) {
+  force(type); force(dpi)
+  function(file, width, height, ...) {
+    grDevices::quartz(file = file, width = width, height = height, type = type, dpi = dpi, ...)
+  }
+}
+
+# a wrapper of the tikzDevice::tikz device
+tikz_dev = function(...) {
+  loadNamespace('tikzDevice')
+  packages = switch(
+    getOption('tikzDefaultEngine'),
+    pdftex = getOption('tikzLatexPackages'),
+    xetex = getOption('tikzXelatexPackages'),
+    luatex = getOption('tikzLualatexPackages')
+  )
+  tikzDevice::tikz(..., packages = c('\n\\nonstopmode\n', packages, .knitEnv$tikzPackages))
+}
+
+# a wrapper of the ragg::agg_png device
+ragg_png_dev = function(...) {
+  loadNamespace('ragg')
+  args = list(...)
+  # handle bg -> background gracefully
+  args$background = args$background %n% args$bg
+  args$bg = NULL
+  do.call(ragg::agg_png, args)
+}
+
+# save a recorded plot
+save_plot = function(plot, name, dev, width, height, ext, dpi, options) {
+
+  path = paste(name, ext, sep = '.')
+  # when cache=2 and plot file exists, just return the filename
+  if (options$cache == 2 && cache$exists(options$hash, options$cache.lazy)) {
+    if (in_base_dir(!file.exists(path))) {
+      purge_cache(options)
+      stop('cannot find ', path, '; the cache has been purged; please re-compile')
+    }
+    return(paste(name, if (dev == 'tikz' && options$external) 'pdf' else ext, sep = '.'))
+  }
+  device = dev_get(dev, options, dpi)
   in_base_dir(plot2dev(plot, name, dev, device, path, width, height, options))
 }
 
@@ -218,7 +234,7 @@ is_low_change = function(p1, p2) {
 
 # recycle some plot options such as fig.cap, out.width/height, etc when there
 # are multiple plots per chunk
-.recyle.opts = c('fig.cap', 'fig.scap', 'fig.env', 'fig.pos', 'fig.subcap',
+.recyle.opts = c('fig.cap', 'fig.scap', 'fig.alt', 'fig.env', 'fig.pos', 'fig.subcap',
                  'out.width', 'out.height', 'out.extra', 'fig.link')
 
 # when passing options to plot hooks, reduce the recycled options to scalars
@@ -361,6 +377,12 @@ par2 = function(x) {
     # drawn at (1, 1) instead of (1, 2)
     x$mfg = NULL
   }
+  if (!is.null(x$fg)) {
+    # set fg before the rest of the par because
+    # it resets col to the same value #1603
+    par(fg = x$fg)
+    x$fg = NULL
+  }
   # you are unlikely to want to reset these pars
   x$fig = x$fin = x$pin = x$plt = x$usr = NULL
   x$ask = NULL  # does not make sense for typical non-interactive R sessions
@@ -412,7 +434,9 @@ include_graphics = function(
     i = file.exists(path2)
     path[i] = path2[i]
   }
-  if (error && length(p <- path[!is_web_path(path) & !file.exists(path)])) stop(
+  # relative paths can be tricky in child documents, so don't error (#1957)
+  if (child_mode()) error = FALSE
+  if (error && length(p <- path[!xfun::is_web_path(path) & !file.exists(path)])) stop(
     'Cannot find the file(s): ', paste0('"', p, '"', collapse = '; ')
   )
   structure(path, class = c('knit_image_paths', 'knit_asis'), dpi = dpi)
@@ -449,10 +473,10 @@ raster_dpi_width = function(path, dpi) {
 #' the output. \code{include_app()} takes the URL of a Shiny app and adds
 #' \samp{?showcase=0} to it (to disable the showcase mode), then passes the URL
 #' to \code{include_url()}.
-#' @param url Character string containing a URL.
-#' @param height Character string with the height of the iframe.
+#' @param url A character vector of URLs.
+#' @param height A character vector to specify the height of iframes.
 #' @return An R object with a special class that \pkg{knitr} recognizes
-#'   internally to generate the iframe or screenshot.
+#'   internally to generate the iframes or screenshots.
 #' @seealso \code{\link{include_graphics}}
 #' @export
 include_url = function(url, height = '400px') {
@@ -470,7 +494,8 @@ include_url2 = function(url, height = '400px', orig = url) {
 #' @export
 include_app = function(url, height = '400px') {
   orig = url  # store the original URL
-  if (!grepl('?', url, fixed = TRUE)) url = paste0(url, '?showcase=0')
+  i = !grepl('?', url, fixed = TRUE)
+  url[i] = paste0(url[i], '?showcase=0')
   include_url2(url, height, orig)
 }
 
@@ -488,7 +513,7 @@ need_screenshot = function(x, ...) {
   if (length(fmt) == 0 || force) return(i1 || i2 || i3)
   html_format = fmt %in% c('html', 'html4', 'html5', 'revealjs', 's5', 'slideous', 'slidy')
   res = ((i1 || i3) && !html_format) || (i2 && !(html_format && runtime_shiny()))
-  res && webshot_available()
+  res && any(webshot_available())
 }
 
 runtime_shiny = function() {
@@ -496,10 +521,18 @@ runtime_shiny = function() {
 }
 
 webshot_available = local({
-  res = NULL  # cache the availablity of webshot/PhantomJS
+  res = NULL  # cache the availability of webshot2/Chrome and webshot/PhantomJS
+  test = function(pkg, fun, pkg2 = pkg) {
+    loadable(pkg) && tryCatch(
+      file.exists(getFromNamespace(fun, pkg2)()),
+      error = function(e) FALSE
+    )
+  }
   function() {
-    if (is.null(res))
-      res <<- loadable('webshot') && !is.null(getFromNamespace('find_phantom', 'webshot')())
+    if (is.null(res)) res <<- c(
+      webshot2 = test('webshot2', 'find_chrome', 'chromote'),
+      webshot  = test('webshot',  'find_phantom')
+    )
     res
   }
 })
@@ -527,6 +560,8 @@ html_screenshot = function(x, options = opts_current$get(), ...) {
   if (is.null(wargs$delay)) wargs$delay = if (i1) 0.2 else 1
   d = tempfile()
   dir.create(d); on.exit(unlink(d, recursive = TRUE), add = TRUE)
+  w = webshot_available()
+  webshot = c(options$webshot, names(w)[w])[[1L]]
   f = in_dir(d, {
     if (i1 || i3) {
       if (i1) {
@@ -534,19 +569,21 @@ html_screenshot = function(x, options = opts_current$get(), ...) {
         save_widget(x, f1, FALSE, options = options)
       } else f1 = x$url
       f2 = wd_tempfile('webshot', ext)
-      do.call(webshot::webshot, c(list(f1, f2), wargs))
-      normalizePath(f2)
+      f3 = do.call(getFromNamespace('webshot', webshot), c(list(f1, f2), wargs))
+      normalizePath(f3)
     } else if (i2) {
-      f = wd_tempfile('webshot', ext)
-      do.call(webshot::appshot, c(list(x, f), wargs))
-      normalizePath(f)
+      f1 = wd_tempfile('webshot', ext)
+      f2 = do.call(getFromNamespace('appshot', webshot), c(list(x, f1), wargs))
+      normalizePath(f2)
     }
   })
-  res = readBin(f, 'raw', file.info(f)[, 'size'])
-  structure(
-    list(image = res, extension = ext, url = if (i3) x$url.orig),
-    class = 'html_screenshot'
-  )
+  lapply(f, function(filename) {
+    res = xfun::read_bin(filename)
+    structure(
+      list(image = res, extension = ext, url = if (i3) x$url.orig[filename == f]),
+      class = 'html_screenshot'
+    )
+  })
 }
 
 save_widget = function(..., options) {
